unit QGrey;

interface

uses
  SysUtils, QGraphics, Classes, QControls, QExtCtrls, UStandard;

type
  TSelectEvent = procedure (nr,grey: integer; color: TColor) of object;
  TGreyscale = class(TImage)
  private
    FScale: integer;       // number of current brush
    FColor: TColor;
    FGrey: integer;
    FOnSelect: TSelectEvent;
    FMarkPos: integer;

    procedure SetScale (nr: integer);
    procedure ResetScale;
    procedure PaintScaleFrame (nr: integer; active: boolean);
    function  FindScaleButton (x,y: integer; var row: integer): integer;
    procedure PaintMark (active: boolean);
  protected
    procedure MouseDown (Button: TMouseButton; Shift: TShiftState; x, y: integer); override;
    procedure MouseMove (Shift: TShiftState; x, y: integer); override;
    procedure MouseUp (Button: TMouseButton; Shift: TShiftState; x, y: integer); override;
  public
    constructor Create (AOwner: TComponent); override;
    destructor  Destroy; override;
    procedure   Init;
    procedure   SetMark (grey: integer);
    procedure   ResetMark;
    procedure   SetGrey (nr: integer);
  published
    property  OnSelect: TSelectEvent read FOnSelect write FOnSelect;
    property  Scale: integer read FScale;
    property  Color: TColor read FColor;
    property  Grey: integer read FGrey;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Beispiele', [TGreyscale]);
end;

procedure TGreyScale.SetGrey; {--------------------------------}
begin
  SetScale (nr);
end;

procedure TGreyscale.Init; {-----------------------------------}
begin
  FScale := 18;
  FColor := clBlack;
  FGrey := 0;
  PaintScaleFrame (18, true);
  Visible := true;
end;

constructor TGreyscale.Create; {=============================}
var i,idx: integer;
    ss: String;
begin
  inherited Create (AOwner);
  AutoSize := true;
  FMarkPos := -1;
end;

destructor TGreyscale.Destroy; {=============================}
begin
  inherited Destroy;
end;

procedure TGreyscale.MouseDown; {============================}
var nr,row:integer;
begin
//  inherited MouseDown(Button, Shift, x,y);
  nr := FindScaleButton (x, y, row);
  if (nr > 0) and (nr < 19) then begin
    SetScale (nr);
    if Assigned (OnSelect) then OnSelect (nr, FGrey,FColor);
  end;
end;

procedure TGreyscale.MouseMove; {============================}
var row,nr:integer;
begin
  inherited MouseMove(Shift, x,y);
  nr:=FindScaleButton(x,y,row);
end;

procedure TGreyscale.MouseUp; {==============================}
begin
//  inherited MouseUp(Button, Shift, x,y);
end;

procedure TGreyscale.SetScale; {------------------------------}
begin
  ResetScale;
  FScale := nr;
  if FScale>0 then PaintScaleFrame (nr,true);
end;

procedure TGreyscale.ResetScale; {----------------------------}
begin
  if FScale < 1 then exit;
  PaintScaleFrame (FScale,false);
  FScale := 0;
end;

procedure TGreyscale.PaintScaleFrame; {-----------------------}
var x,y:integer;
begin
  y := (nr-1) * 19 + 9;
  x := 22;
  with Canvas do begin
    pen.width := 2;
    if active then pen.color := clYellow else pen.color := $00ffb478;
    brush.Style := bsClear;
    rectangle (x, y, x+33, y+18);
  end;
end;

function TGreyscale.FindScaleButton; {------------------------}
begin
  row := (y + 7) div 19;
  if y > (row-1) * 19 + 24 then row := 0;
  if row > 18 then row := 0;
  if (x < 24) or (x > 54) then row := 0;
  if row > 0 then begin
    Result := row;
    FGrey := (18 - row) * 15;
    FColor := cCol (FGrey, FGrey, FGrey);
  end else begin
    Result := 0;
  end;
end;


procedure TGreyscale.SetMark; {--------------------------------}

begin

  ResetMark;

  FMarkPos := 339 - round (grey * 1.264);

  PaintMark (true);

end;


procedure TGreyscale.ResetMark; {------------------------------}

begin

  if FMarkPos < 0 then exit;

  PaintMark (false);

  FMarkPos := -1;

end;

procedure TGreyscale.PaintMark; {------------------------------}
begin
  if active then Canvas.Pen.Color := clWhite else Canvas.Pen.Color := $00ffaa55;
  Canvas.MoveTo (4, FMarkPos);
  Canvas.LineTo (16, FMarkPos);
  Canvas.MoveTo (4, FMarkPos + 1);
  Canvas.LineTo (16, FMarkPos + 1);
  Canvas.MoveTo (4, FMarkPos + 2);
  Canvas.LineTo (16, FMarkPos + 2);
end;

end.


