unit QTerrBrush;

interface

uses
  SysUtils, QGraphics, Classes, QControls, QExtCtrls, UStandard;

type
  TSelectEvent = procedure (nr: integer) of object;
  TTerrBrush = class(TImage)
  private
    FBrush: integer;       // number of current brush
    FOnSelect: TSelectEvent;

    procedure SetBrush (nr:integer);
    procedure ResetBrush;
    procedure PaintBrushFrame (nr:integer; active: boolean);
    function  FindBrushButton (x,y:integer; var row,col:integer): integer;
  protected
    procedure MouseDown (Button: TMouseButton; Shift: TShiftState; x, y: integer); override;
    procedure MouseMove (Shift: TShiftState; x, y: integer); override;
    procedure MouseUp (Button: TMouseButton; Shift: TShiftState; x, y: integer); override;
  public
    constructor Create (AOwner:TComponent); override;
    destructor  Destroy; override;
    procedure   Init;
  published
    property  OnSelect: TSelectEvent read FOnSelect write FOnSelect;
    property  Brush: integer read FBrush write SetBrush;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Beispiele', [TTerrBrush]);
end;

procedure TTerrBrush.Init;
begin
  FBrush := 1;
  PaintBrushFrame(1,true);
  Visible := true;
end;

constructor TTerrBrush.Create; {=============================}
var i,idx: integer;
    ss: String;
begin
  inherited Create (AOwner);
  AutoSize:=true;
end;

destructor TTerrBrush.Destroy; {=============================}
begin
  inherited Destroy;
end;

procedure TTerrBrush.MouseDown; {============================}
var nr,row,col:integer;
begin
//  inherited MouseDown(Button, Shift, x,y);
  nr:=FindBrushButton(x,y,row,col);
  if nr<>0 then begin
    SetBrush(nr);
    if Assigned (OnSelect) then OnSelect (nr);
  end;
end;

procedure TTerrBrush.MouseMove; {============================}
var row,col,nr:integer;
begin
  inherited MouseMove(Shift, x,y);
  nr:=FindBrushButton(x,y,row,col);
  if nr<>0 then Cursor:=crHandPoint else Cursor := crDefault;
end;

procedure TTerrBrush.MouseUp; {==============================}
begin
//  inherited MouseUp(Button, Shift, x,y);
end;

procedure TTerrBrush.SetBrush; {------------------------------}
begin
  ResetBrush;
  FBrush := nr;
  PaintBrushFrame (nr,true);
end;

procedure TTerrBrush.ResetBrush; {----------------------------}
begin
  if FBrush < 1 then exit;
  PaintBrushFrame (FBrush,false);
  FBrush := 0;
end;

procedure TTerrBrush.PaintBrushFrame; {-----------------------}
var x,y:integer;
begin
  y:= ((nr-1) div 8) * 30 + 6;
  x:= ((nr-1) mod 8) * 30 + 6;
  with Canvas do begin
    pen.width:=2;
    if active then pen.color:=clRed else pen.color:= $00e4e4e4;  //$00ffb478;
    brush.Style:=bsClear;
    rectangle (x,y,x+24,y+24);
  end;
end;

function TTerrBrush.FindBrushButton; {------------------------}
begin
  row:=(y+25) div 30;
  if y>(row-1)*30 + 30 then row:=0;
  col:=(x+25) div 30;
  if x>(col-1)*30 + 30 then col:=0;
  if (col>0) and (row>0) then result:=(row-1)*8 + col else result:=0;
end;



end.


