unit QTerrPalette;

interface

uses
  SysUtils, QGraphics, Classes, QControls, QExtCtrls, UStandard,
  UTuxBase;

type
  TSelectEvent = procedure (Color: TColor; TerrName: String) of object;
  TTerrPalette = class(TImage)
  private
    FTerrain: integer;       // number of current terrain
    FOnSelect: TSelectEvent;
    FList: TStringList;      // terrains.lst
    FTypIdx: String;
    FColorIdx: String;
    FColor: TColor;          // current terrain color
    FTerrName: String;       // current terrain name

    procedure SetTerr (nr:integer);
    procedure ResetTerr;
    procedure PaintTerrFrame (nr:integer; active: boolean);
    function  FindTerrButton (x,y:integer; var row,col:integer): integer;
  protected
    procedure MouseDown (Button: TMouseButton; Shift: TShiftState; x, y: integer); override;
    procedure MouseMove (Shift: TShiftState; x, y: integer); override;
    procedure MouseUp (Button: TMouseButton; Shift: TShiftState; x, y: integer); override;
  public
    constructor Create (AOwner:TComponent); override;
    destructor  Destroy; override;
    procedure   Init;
    function    Col_TerrName (col: TColor): String;
  published
    property  OnSelect: TSelectEvent read FOnSelect write FOnSelect;
    property  Terrain: integer read FTerrain write FTerrain;
    property  Color: TColor read FColor write FColor;
    property  TerrName: String read FTerrName write FTerrName;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Beispiele', [TTerrPalette]);
end;

function TTerrPalette.Col_TerrName;
var idx: integer;
    ss: String;
begin
  idx := ParsInt (FColorIdx, ColorToString (col), 0);
  ss := FList [idx];
  Result := ParsStr(ss, 'terr');
end;

procedure TTerrPalette.Init;
var i: integer;
    ss: String;
begin
  // load terrain.lst
  if FileExists (ProgDir+'res'+SEP+'terrains.lst') then begin
    FList.LoadFromFile(ProgDir+'res'+SEP+'terrains.lst');
    FTypIdx:='';
    FColorIdx:='';
    // create typ-index and color-index
    for i:=0 to FList.Count-1 do begin
      ss:=FList[i];
      FTypIdx:=FTypIdx+'['+ParsStr(ss,'nr')+']'+IntToStr(i);
      FColorIdx:=FColorIdx+'['+ParsStr(ss,'col')+']'+IntToStr(i);
    end;
    iiTerrlistOK := true;
  end else begin
    iiTerrlistOK:= false;
  end;

  // set default values
  if iiTerrlistOK then begin
    FTerrain := 1;
    FColor := clWhite;
    FTerrName := 'snow';
    PaintTerrFrame (1,true);
    Visible := true;
  end else Visible := false;
end;

constructor TTerrPalette.Create; {=============================}
var i,idx: integer;
    ss: String;
begin
  inherited Create (AOwner);
  FList := TStringList.Create;
  AutoSize:=true;
end;

destructor TTerrPalette.Destroy; {=============================}
begin
  FList.Free;
  inherited Destroy;
end;

procedure TTerrPalette.MouseDown; {============================}
var nr,row,col:integer;
begin
//  inherited MouseDown(Button, Shift, x,y);
  nr:=FindTerrButton(x,y,row,col);
  if nr<>0 then SetTerr(nr);
end;

procedure TTerrPalette.MouseMove; {============================}
var row,col,nr:integer;
begin
  inherited MouseMove(Shift, x,y);
  nr:=FindTerrButton(x,y,row,col);
  if nr<>0 then Cursor:=crHandPoint else Cursor := crDefault;
end;

procedure TTerrPalette.MouseUp; {==============================}
begin
//  inherited MouseUp(Button, Shift, x,y);
end;

procedure TTerrPalette.SetTerr; {------------------------------}
var idx:integer;
    ss:String;
begin
  ResetTerr;
  FTerrain := nr;
  PaintTerrFrame (nr,true);
  idx := ParsInt (FTypIdx,IntToStr(nr), 0);
  ss := FList[idx];
  FColor := StringToColor (ParsStr (ss, 'col'));
  FTerrName := ParsStr (ss, 'terr');
  if Assigned (OnSelect) then OnSelect (FColor, FTerrName);
end;

procedure TTerrPalette.ResetTerr; {----------------------------}
begin
  if FTerrain < 1 then exit;
  PaintTerrFrame (FTerrain,false);
  FTerrain := 0;
end;

procedure TTerrPalette.PaintTerrFrame; {-----------------------}
var x,y:integer;
begin
  y:= ((nr-1) div 7) * 64 + 5;
  x:= ((nr-1) mod 7) * 56 + 6;
  with Canvas do begin
    pen.width:=4;
    if active then pen.color:=clRed else pen.color:=$00ffb478;
    brush.Style:=bsClear;
    rectangle (x,y,x+46,y+46);
  end;
end;

function TTerrPalette.FindTerrButton; {------------------------}
begin
  row:=(y+56) div 64;
  if y>(row-1)*64+48 then row:=0;
  col:=(x+47) div 56;
  if x>(col-1)*56+49 then col:=0;
  if (col>0) and (row>0) then result:=(row-1)*7 + col
  else result:=0;
end;



end.


