unit UFiles;

interface

uses
  QGraphics, Types, SysUtils, Libc, Classes, Math, QDialogs, UTuxbase,
  QExtCtrls, UStandard;

var
  {lokal fr den Aufbau von items.tcl, procedurbergreifend}
  ObjNr: integer;  // Fortlaufende Nummerierung der Objektbezeichnungen
  spFlag: boolean; // Flag, dass am Schluss Startpoints angehngt werden mssen
  xsp, ysp, zsp: double;


procedure F_LoadBitmaps;
procedure F_SaveElev;
procedure F_SaveTerr;

{--- lines of items.tcl ---}
function  TCLTopLine (name: String; node: String): String;
function  TCLObjectLine (name: String): String;
function  TCLPositionLine (px,py,pz: double): String;
function  TCLScaleLine (sx,sy,sz: double): String;
function  TCLRotationLine (rx,ry,rz: double): String;
function  TCLShadowLine (val: integer): String;
procedure TCLAddItemHeader (li: TStringList);
procedure TCLAddStartPoints (li: TStringList);

{... course.tcl ---}
procedure F_LoadCourseTCL;
procedure F_SaveCourseTCL;

{--- objects (objects.arr and items.tcl ---}
procedure F_LoadObjects;    // objects.arr or items.tcl
procedure F_SaveObjects;    // objects.arr
procedure F_LoadItemsTCL;   // render items.tcl
procedure F_SaveItemsTCL;
procedure F_AddItem (obj: String; li: TStringList);

implementation

function TCLTopLine; {-----------------------------------------}
var ss: String;
begin
  inc (ObjNr);
  if node = 'b' then begin
    ss := 'objcreate {s_object3dinst} {:scene:basenode:'
      + name + '-' + IntToStr(ObjNr) + '} \';
  end else if node='s' then begin
    ss := 'objcreate {s_object3dinst} {:start_points:'
      + name + '-' + IntToStr(ObjNr) + '} \';
  end else if node='r' then begin
    ss:='objcreate {s_object3dinst} {:reset_points:'
      + name + '-' + IntToStr(ObjNr) + '} \';
  end;
  Result := ss;
end;

function TCLObjectLine; {--------------------------------------}
begin
  Result := chr(9) + '{-object3d} {:objects:' + name + '} \';
end;

function TCLPositionLine; {------------------------------------}
begin
  Result := chr(9) + '{-position} {'
    + Float_Str (px, 4) + ' ' + Float_Str (pz, 8) + ' '
    + Float_Str (py, 4) + '}';
end;

function TCLRotationLine; {------------------------------------}
begin
  Result := chr(9) + '{-rotation} {'
    + Float_Str (ry, 4) + ' ' + Float_Str (rz, 4) + ' '
    + Float_Str (rx, 4) + '}';
end;

function TCLScaleLine; {---------------------------------------}
begin
  Result := chr(9)+'{-scale} {'
    + Float_Str (sx, 4) + ' ' + Float_Str (sz, 4) + ' '
    + Float_Str (sy, 4) + '}';
end;

function TCLShadowLine; {--------------------------------------}
begin
  Result := chr(9) + '{cast_shadow} {' + IntToStr(val) + '}';
end;

procedure TCLAddItemHeader; {----------------------------------}
begin
  li.Add ('# generated by Tuxeditor');
  li.Add ('');
  li.add ('objcreate {s_sgnode} {:scene}');
  li.add ('objcreate {s_sgnode} {:scene:basenode}');
  li.add ('objcreate {s_sgnode} {:reset_points}');
  li.add ('');
end;

procedure TCLAddStartpoints; {---------------------------------}
begin
  li.add ('objcreate {s_sgnode} {:start_points}');
  li.add ('');
  li.Add ('objcreate {s_object3dinst} {start_points:start_point} \');
  li.Add (chr(9) + '{-object3d} {:objects:start_point} \');
  li.Add (TCLPositionLine (xsp - 0.9, ysp + 0.5, zsp + 1.3));
  li.Add ('');
  li.Add ('objcreate {s_object3dinst} {start_points:start_point-1} \');
  li.Add (chr(9) + '{-object3d} {:objects:start_point} \');
  li.Add (TCLPositionLine (xsp + 0.9, ysp + 0.5, zsp + 1.3));
  li.Add ('');
  li.Add ('objcreate {s_object3dinst} {start_points:start_point-2} \');
  li.Add (chr(9) + '{-object3d} {:objects:start_point} \');
  li.Add (TCLPositionLine (xsp - 2.7, ysp + 0.5, zsp + 1.3));
  li.Add ('');
  li.Add ('objcreate {s_object3dinst} {start_points:start_point-3} \');
  li.Add (chr(9) + '{-object3d} {:objects:start_point} \');
  li.Add (TCLPositionLine (xsp + 2.7, ysp + 0.5, zsp + 1.3));
  li.Add ('');
end;

procedure F_LoadBitmaps; {-------------------------------------}
// iiTrackdir must already be set
begin
  if FileExists (B_ElevFile) then EMap.LoadFromFile (B_ElevFile);
  if FileExists (B_TerrainFile) then TMap.LoadFromFile (B_TerrainFile);
end;

procedure F_SaveElev; {----------------------------------------}
begin
  if not iiOpen then exit;
  if iiElevChanged then begin
    EArr_EMap;
    EMap.SaveToFile (B_ElevFile);
    iiElevChanged := false;
  end;
end;

procedure F_SaveTerr; {------------------------------------}
begin
  if not iiOpen then exit;
  if iiTerrChanged then begin
    TMap.SaveToFile (B_TerrainFile);
    iiTerrChanged := false;
  end;
end;


procedure F_LoadCourseTCL; {-----------------------------------}
var co: TStringList;
    i, po: integer;
    ss, val: String;
begin
  if FileExists (B_CourseFile) then begin
    co := TStringList.Create;
    co.LoadFromFile (B_CourseFile);
    for i := 0 to co.Count-1 do begin
      ss := co[i];
      po := pos ('-length', ss);
      if po > 0 then iiCourseLength := StrToFloat (BareStr (copy (ss, po+7, 100)));
      po := pos ('-width', ss);
      if po > 0 then iiCourseWidth := StrToFloat (BareStr (copy (ss, po+6, 100)));
      po := pos ('-play_length', ss);
      if po>0 then iiPlayLength := StrToFloat (BareStr (copy (ss, po+12, 100)));
      po := pos ('-play_width', ss);
      if po > 0 then iiPlayWidth := StrToFloat (BareStr (copy( ss, po+11, 100)));
      po := pos ('-angle', ss);
      if po > 0 then iiAngle := StrToFloat (BareStr (copy (ss, po+6, 100)));
      po := pos ('-elev_scale', ss);
      if po > 0 then iiScale := StrToFloat (BareStr (copy (ss, po+11, 100)));
      po := pos ('-start_pt', ss);
      if po > 0 then begin
        val := BareStr (copy (ss, po+9, 100));
        iiStartStr := val;
      end;
      po := pos ('-course_name', ss);
      if po > 0 then begin
        val := BareStr (copy (ss, po+12, 100));
        iiCourseName := copy (val, 2, length(val) - 2);
      end;
    end;
    co.Free;
  end else begin // course.tcl does not exist: set bitmap-related params
    iiCourseName := iiTrackName;
    iiCourseLength := EMap.Height;
    iiCourseWidth := EMap.Width;
    iiPlayLength := iiCourseLength - 50;
    iiPlayWidth := iiCourseWidth / 1.1;
    iiAngle := 24;
    iiScale := 20;
  end;
end;

procedure F_SaveCourseTCL; {-----------------------------------}
var co: TStringList;
begin
  if not iiOpen then exit;
  if iiCourseChanged then begin
    co := TStringList.Create;
    co.Add ('# generated by Tuxeditor');
    co.Add ('');
    co.Add ('objset :course -course_name "' + iiCourseName + '"');
    co.Add ('objset :course:heightmap -width ' + IntToStr (trunc (iiCourseWidth)));
    co.Add ('objset :course:heightmap -length ' + IntToStr (trunc (iiCourseLength)));
    co.Add ('objset :course:heightmap -play_width ' + IntToStr (trunc (iiPlayWidth)));
    co.Add ('objset :course:heightmap -play_length ' + IntToStr (trunc (iiPlayLength)));
    co.Add ('objset :course:heightmap -angle ' + IntToStr (trunc (iiAngle)));
    co.Add ('objset :course:heightmap -elev_scale ' + IntToStr (trunc (iiScale)));
    co.Add ('objcall :course:heightmap load_elevation elev.png');
    co.Add ('');
    co.Add ('objset :course -start_pt ' + iiStartStr);
    co.Add ('');
    co.Add ('tux_course_init');
    co.SaveToFile (B_CourseFile);
    co.Free;
    iiCourseChanged := false;
  end;
end;

procedure F_LoadObjects; {-------------------------------------}
var ok: boolean;
begin
  if FileExists (B_OArrFile) then begin // first try to load OArr
    OArr.LoadFromFile (B_OArrFile);
    ok := true;
  end else begin  // if failed then render items.tcl
    OArr.Clear;
    if FileExists( B_ItemsFile) then begin
      F_LoadItemsTCL;
      ok := true;
    end else ok := false;
  end;
  if ok then B_MakeOIdx;
end;

procedure F_SaveObjects; {-------------------------------------}
var fi: TStringList;
    i: integer;
begin
  if not iiOpen then exit;
  if iiObjChanged or iiCourseChanged then begin
    fi := TStringList.Create;
    if OArr.Count > 0 then for i := 0 to OArr.Count-1 do begin
      if OArr[i] <> 'Del' then fi.Add (OArr[i]);
    end;
    fi.SaveToFile (B_OArrFile);
    fi.Free;
    F_SaveItemsTCL;
    iiObjChanged := false;
  end;
end;

procedure F_LoadItemsTCL; {------------------------------------}
var bmx, bmy, bmz, idx, i, po:integer;
    RotStr, PosStr, ScaleStr, name, a, b: String;
    bmxEx, dx, dy, dz, ex, ey, ez, x, y, z:double;
    list1, list2: TStringList;

procedure SelectParams (par: String); {........................}
var anz: integer;
begin
  a := ' '+ BareStr (ParsStr (b, par));
  if a <> ' ' then begin
     anz := 0;
     repeat
       po := pos (' ',a);
       if po > 0 then begin
         inc (anz);
         delete (a, po, 1);
         insert ('{' + IntToStr(anz) + '}', a, po);
       end;
     until po = 0;
     SetParsStr (b, par, a);
  end;
end;

begin {........................................................}

  list1 := TStringList.Create;
  list2 := TStringList.Create;
  list1.LoadFromFile (B_ItemsFile);

  // delete tabs and spaces at TOL and EOL:
  for i := 0 to list1.Count-1 do list1[i] := BareStr (list1[i]);

  // join lines which belong together:
  for i := list1.Count-1 downto 1 do begin
    a := list1[i-1];
    b := list1[i];
    if (a <> '') and (b <> '') then begin
      if a[length(a)] = '\' then begin
        delete (a, length(a), 1);
        list1[i-1] := a + b;
        list1[i] := '';
      end;
    end;
  end;

  // delete empty lines:
  for i := list1.count-1 downto 0 do if list1[i] = '' then list1.delete(i);

  // delete '\' at EOL if exists (see tuxcourser)
  for i := 0 to list1.count-1 do begin
    a := list1[i];
    if a[length(a)] = '\' then delete (a, length(a), 1);
    list1[i] := a;
  end;

  // copy 3dobjects to list2
  for i := 0 to list1.Count-1 do begin
    if pos ('{s_object3dinst}', list1[i]) <> 0 then list2.Add (list1[i]);
  end;

  // alles vor dem Namen lschen:
  for i := 0 to list2.Count-1 do begin
    po := pos (':objects:', list2[i]);
    if po <> 0 then list2[i] := copy (list2[i], po+9, 200);
  end;

  // in Standard-Pars-Tags umwandeln:
  for i := 0 to list2.Count-1 do begin
    a := list2[i];
    po := pos ('}', a);
    if po <> 0 then Delete (a, po, 1);
    a := changepart (a, '{-position}', '[-position]');
    a := changepart (a, '{-rotation}', '[-rotation]');
    a := changepart (a, '{-scale}', '[-scale]');
    a := changepart (a, '{-cast_shadow}', '[-cast_shadow]');
    while pos ('{', a) <> 0 do delete (a, pos('{', a), 1);
    while pos ('}', a) <> 0 do delete (a, pos('}', a), 1);
    list2[i] := '[name]' + a;
  end;

  // Einfgen der Parameternummerierung {1}{2}{3}
  for i := 0 to list2.Count-1 do begin
    b := list2[i];
    SelectParams ('-position');
    SelectParams ('-scale');
    SelectParams ('-rotation');
    list2[i] := b;
  end;
  // Ende der Vorbereitungen

  // Beginn der Auswertung
  OArr.Clear;
  for i := 0 to list2.Count-1 do begin
    a := list2[i];
    name := ParsStr (a, 'name');
    idx := ParsInt (ONameIdx, name, -1);
    if (idx >= 0) and (idx < OList.Count) then begin
      b := OList[idx];

      PosStr := ParsStr (a, '-position');

      // x-Position auslesen
      x := ParsFloat2 (PosStr, '1', 0);
      ex := B_xPixPos(x);  // exakte Pixelposition (mit Nachkommastellen)
      if ex < 0 then ex := 0;
      if ex > iiWidth-1 then ex := iiWidth-1;
      bmx := round(ex);   // ganzzahlige Pixelposition
      dx := ex - bmx;     // Abweichung von der exakten Position
      SetParsInt (b, 'x', bmx);  // Pixelwert eintragen
      SetParsFloat (b, 'dx', dx, 4);  // X-Abweichung als Verschiebung dx eintragen

      // dasselbe mit der y-Position
      y := ParsFloat2 (PosStr, '3', 0);
      ey := B_yPixPos (y);
      if ey < 0 then ey := 0;
      if ey > iiHeight-1 then ey := iiHeight-1;
      bmy := round (ey);
      dy := ey - bmy;
      SetParsInt (b, 'y', bmy);
      SetParsFloat (b, 'dy', dy, 4);

      // etwas anders die z-Position:
      ez := ParsFloat2 (PosStr, '2', 0);
      bmz := EArr [bmy, bmx];  // Grauwert an ganzzahliger xy-Position auslesen
      z := B_ZPos (bmz, bmy + dy); //
      dz := ez - z;
      SetParsFloat (b, 'dz', dz, 4);  // nur z-Verschiebung eintragen

      // Skalierung auslesen und eintragen
      ScaleStr := ParsStr (a, '-scale');
      x := ParsFloat2 (ScaleStr, '1', 1);
      y := ParsFloat2 (ScaleStr, '3', 1);
      z := ParsFloat2 (ScaleStr, '2', 1);
      if (x <> 1) or (y <> 1) or (z <> 1) then begin
        SetParsFloat (b, 'sx', x, 4);
        SetParsFloat (b, 'sy', y, 4);
        SetParsFloat (b, 'sz', z, 4);
      end;
      SetParsInt (b, 'eq', 0);

      // Rotation auslesen und eintragen
      RotStr := ParsStr (a, '-rotation');
      x := ParsFloat2 (RotStr, '3', 0);
      y := ParsFloat2 (RotStr, '1', 0);
      z := ParsFloat2 (RotStr, '2', 0);
      if (x <> 0) or (y <> 0) or (z <> 0) then begin
        SetParsFloat (b, 'rx', x, 4);
        SetParsFloat (b, 'ry', y, 4);
        SetParsFloat (b, 'rz', z, 4);
      end;
      SetParsInt (b, 'ca', 0);

      // -cast_shadow nicht auslesen ???

      OArr.Add (b);
    end;
  end;
  list1.Free;
  list2.Free;
end;

procedure F_SaveItemsTCL; {-------------------------------------}
var fi: TStringList;
    i: integer;
begin
  ObjNr := 0; // reset current object-identifier, e.g. flag-25
  spFlag := false; // reset startpoint-flag
  fi := TStringList.Create;

  TCLAddItemHeader (fi);
  for i := 0 to OArr.Count-1 do begin
    if OArr[i] <> 'Del' then F_AddItem (OArr[i], fi);
  end;
  if spFlag then TCLAddStartPoints (fi);

  fi.SaveToFile (B_ItemsFile);
  fi.Free;
end;

procedure F_AddItem; {----------------------------------------}
var ca, i, typ, c, sh: integer;
    bmx, bmy, bmz: integer; // Bitmap-Positionen
    dx, dy, dz: double;  // Verschiebung
    x, y, z: double;     // Kurs-Positionen
    sx, sy, sz: double;  // Skalierung
    rx, ry, rz: double;  // Rotation
    node, name:String;
begin
  typ := ParsInt (obj, 'type', 0);
  if typ < 1 then exit;

  // Parameter auslesen:
  bmx := ParsInt (obj, 'x', 0);
  bmy := ParsInt (obj, 'y', 0);
  bmz := cRed (EMap.canvas.pixels[bmx, bmy]);  // Sonderfall: aus Grauwert
  dx := ParsFloat (obj, 'dx', 0);
  dy := ParsFloat (obj, 'dy', 0);
  dz := ParsFloat (obj, 'dz', 0);

  // Kurspositionen berechnen
  x := B_XPos (bmx + dx);
  y := B_YPos (bmy + dy);
  z := B_ZPos (bmz, bmy + dy) + dz;

  name := ParsStr (obj, 'name');
  node := ParsStr (obj, 'node');

  // Startpunkt testen und Positionen procedurbergreifend notieren
  if pos ('[sp]', obj) <> 0 then begin
    spFlag := true;
    xsp := x;
    ysp := y;
    zsp := z;
  end;

  if node = '' then exit;  // e.g. blank startpoint

  li.Add (TCLTopLine (name, ParsStr (Obj, 'node')[1]));
  li.Add (TCLObjectLine (name));
  li.Add (TCLPositionLine (x, y, z));

  // Add scale line if necessary
  if ParsInt (obj, 'scale', 0) > 0 then begin
    sx := ParsFloat (obj, 'sx', 1);
    sy := ParsFloat (obj, 'sy', 1);
    sz := ParsFloat (obj, 'sz', 1);
    // nur eintragen, wenn mindestens ein Wert <> 1
    if (sx <> 1) or (sy <> 1) or (sz <> 1) then begin
      li [li.count-1] := li [li.count-1] + ' \';
      li.Add (TCLScaleLine (sx, sy, sz));
    end;
  end;

  // Add rotation line if necessary
  if ParsInt (obj, 'rot', 0) > 0 then begin
    ca := ParsInt (obj, 'ca', 0);  // auf Kursneigung

    if ca > 0 then ry := -iiAngle else ry := ParsFloat (obj, 'ry', 0);
    rx := ParsFloat (obj, 'rx', 0);
    rz := ParsFloat (obj, 'rz', 0);
    if (rx <> 0) or (ry <> 0) or (rz <> 0) then begin
      li [li.count-1] := li [li.count-1] + ' \';
      li.Add (TCLRotationLine (rx, ry, rz));
    end;
  end;

  // Add shadow line if necessary
  if ParsInt (obj, 'shad', 0) > 0 then begin
    c := ParsInt (obj, 'sh', 0);
    if sh > 0 then begin
      li [li.count-1] := li [li.count-1] + ' \';
      li.Add (TCLShadowLine (sh));
    end;
  end;
  li.Add ('');
end;

end.
