unit UMaps;

interface

uses
  QGraphics, Types, SysUtils, Libc, Classes, Math, QDialogs, UStandard,
  UTuxBase;

var
  mpClientCanvas: TCanvas;

procedure IncElevPixel    (x, y, val: integer);
procedure PaintSpraySpot  (x, y: integer);
procedure PaintSprayTrack (x1, y1, x2, y2, diffx, diffy: integer);

procedure StoreTerr;
procedure RestoreTerr;
procedure ResetTerrStore;

procedure ResetElevStore;
procedure StoreElev;
procedure RestoreElev;

procedure PaintTerrPixel   (x, y, scale: integer; col: TColor);
procedure PaintTerrLine    (x, y, lng: integer);
procedure PaintTerrRect    (x, y, width, height: integer);
procedure PaintTerrSpot    (x, y: integer);
procedure PaintTerrTrack   (x1, y1, x2, y2, diffx, diffy: integer);

procedure PaintBrushTrack  (x1, y1, x2, y2, diffx, diffy: integer);
procedure PaintClientPixel (x, y: integer; col: TColor);
procedure PaintClientLine  (x, y, lng: integer);
procedure PaintClientRect  (x, y, width, height: integer);
procedure PaintArrRect     (x1, y1, x2, y2: integer);
procedure PaintArrLine     (x1, y1, x2: integer);

function  BlurDot          (x, y, intens: integer): integer;
procedure PaintBlurSpot    (x, y: integer);
procedure GenerateBrush    (size, intens: integer; soft: boolean);
procedure GenerateBrightBrush;

procedure PaintBrushSpot   (x, y: integer);  // for elev
procedure PaintBlurTrack   (x1, y1, x2, y2, diffx, diffy: integer);
procedure PaintBrightSpot  (x, y: integer);
procedure PaintBrightTrack (x1, y1, x2, y2, diffx, diffy: integer);

procedure M_RecallMMap;
procedure M_RecallMMapRect (rr: TRect);
procedure M_StoreMMap;     // speichert MMap nach MMap2


implementation

procedure IncElevPixel;
var gr: integer;
    col: TColor;
begin
  if (x >= 0) and (x < iiWidth) and (y >= 0) and (y < iiHeight) then begin
    gr := EArr [y,x];
    if (gr >= mpVall) and (gr <= mpMount) then begin
      gr := gr + val;
      if gr > 255 then gr := 255;
      if gr < 0 then gr := 0;
      EArr [y,x] := gr;
      col := cCol (gr, gr, gr);
      PaintClientPixel (x, y, col);
    end;
  end;
end;

procedure PaintSpraySpot; {--------------------------------}
var i, xpos, ypos, siz: integer;
    val, left, right, top, bott: integer;
    dist: double;
begin
  if mpElevBrush = 1 then siz := 0
  else if mpElevBrush = 2 then siz := 1
  else siz := mpBrushPar.Size div 2 + 1;
  left := x - siz;
  if left < 0 then left := 0;
  right := x + siz;
  if right > iiWidth-1 then right := iiWidth-1;
  top := y - siz;
  if top < 0 then top := 0;
  bott := y + siz;
  if bott > iiHeight-1 then bott := iiHeight-1;

  for i := 1 to siz * siz + 10 do begin
    xpos := XRandom (left, right);
    ypos := XRandom (top, bott);
    val := XRandom (mpMinSpray, mpMaxSpray);
    if mpElevBrush > 8 then begin
      dist := sqrt ((x - xpos) * (x - xpos) + (y - ypos) * (y - ypos));
      if dist < 1 then dist := 1;
      val := round (XRandom (mpMinSpray, mpMaxSpray) / dist * 2);
    end else begin
      val := XRandom (mpMinSpray, mpMaxSpray);
    end;
    IncElevPixel (xpos, ypos, val);
  end;
end;

procedure ResetTerrStore;
var i: integer;
begin
  for i := 1 to MaxTerrStore do TerrPoint[i] := i;
  TerrCount := 0;
end;

procedure ResetElevStore;
var i: integer;
begin
  for i := 1 to MaxElevStore do ElevPoint[i] := i;
  ElevCount := 0;
end;

procedure StoreTerr;
var i, last: integer;
begin
  last := TerrPoint[MaxTerrStore];
  for i := MaxTerrStore downto 2 do TerrPoint[i] := TerrPoint[i-1];
  TerrPoint[1] := last;
  if TerrCount < MaxTerrStore then inc (TerrCount);

  TerrStore[TerrPoint[1]].Height := TMap.Height;
  TerrStore[TerrPoint[1]].Width := TMap.Width;
  TerrStore[TerrPoint[1]].Assign (TMap);
end;

procedure StoreElev; {------------------------------------}
var i, x, y, last: integer;
begin
  last := ElevPoint[MaxElevStore];
  for i := MaxElevStore downto 2 do ElevPoint[i] := ElevPoint[i-1];
  ElevPoint[1] := last;
  if ElevCount < MaxElevStore then inc (ElevCount);

  for y := 0 to iiHeight-1 do for x := 0 to iiWidth-1 do begin
    ElevStore[ElevPoint[1]][y,x] := EArr[y,x];
  end;
end;

procedure RestoreTerr;
var i, first: integer;
begin
  if TerrCount < 1 then exit;
  TMap.Assign (TerrStore[TerrPoint[1]]);
  first := TerrPoint[1];
  for i := 1 to MaxTerrStore-1 do TerrPoint[i] := TerrPoint[i+1];
  TerrPoint[MaxTerrStore] := first;
  dec (TerrCount);
end;

procedure RestoreElev; {----------------------------------}
var i, x, y, first: integer;
begin
  if ElevCount < 1 then exit;

  for y := 0 to iiHeight-1 do for x := 0 to iiWidth-1 do begin
    EArr[y,x] := ElevStore[ElevPoint[1]][y,x];
  end;

  first := ElevPoint[1];
  for i := 1 to MaxElevStore-1 do ElevPoint[i] := ElevPoint[i+1];
  ElevPoint [MaxElevStore] := first;
  dec (ElevCount);
end;

procedure PaintTerrPixel; {------------------------------------}
var xx, yy: integer;
begin
  if scale = 1 then begin
    TMap.Canvas.Pixels[x,y] := col;
  end else begin
    TMap.Canvas.brush.color := col;
    TMap.Canvas.pen.color := col;
    xx := x * scale;
    yy := y * scale;
    TMap.Canvas.rectangle (xx, yy, xx + scale, yy + scale);
  end;
end;

procedure PaintTerrLine; {------------------------------------}
begin
  TMap.Canvas.MoveTo (x, y);
  TMap.Canvas.LineTo (x + lng, y);
end;

procedure PaintTerrRect; {-------------------------------------}
begin
  TMap.Canvas.Rectangle (x, y, x + width, y + height);
end;

procedure PaintTerrSpot; {-------------------------------------}
begin
  if iiOpen = false then exit;

  // set client-canvas color
  mpClientCanvas.Brush.Color := mpTerrCol;
  mpClientCanvas.Pen.Color := mpTerrCol;
  TMap.Canvas.Brush.Color := mpTerrCol;
  TMap.Canvas.Pen.Color := mpTerrCol;

  case mpTerrBrush of
    1: begin
      PaintClientPixel (x, y, mpTerrCol);
      PaintTerrPixel (x, y, 1, mpTerrCol);
      end;
    2: begin
      PaintClientRect (x-1, y-1, 2, 2);
      PaintTerrRect (x-1, y-1, 2, 2);
      end;
    3: begin
      PaintClientRect (x-1, y-1, 3, 3);
      PaintTerrRect (x-1, y-1, 3, 3);
      end;
    4: begin
      PaintClientRect (x-2, y-1, 5, 3);
      PaintClientLine (x-1, y-2, 3);
      PaintClientLine (x-1, y+2, 3);
      PaintTerrRect (x-2, y-1, 5, 3);
      PaintTerrLine (x-1, y-2, 3);
      PaintTerrLine (x-1, y+2, 3);
      end;
    5: begin
      PaintClientRect(x-3,y-1,7,3);
      PaintClientLine(x-2,y-2,5);
      PaintClientLine(x-2,y+2,5);
      PaintClientLine(x-1,y-3,3);
      PaintClientLine(x-1,y+3,3);
      PaintTerrRect(x-3,y-1,7,3);
      PaintTerrLine(x-2,y-2,5);
      PaintTerrLine(x-2,y+2,5);
      PaintTerrLine(x-1,y-3,3);
      PaintTerrLine(x-1,y+3,3);
      end;
    6: begin
      PaintClientRect(x-4,y-2,9,5);
      PaintClientLine(x-3,y-3,7);
      PaintClientLine(x-3,y+3,7);
      PaintClientLine(x-2,y-4,5);
      PaintClientLine(x-2,y+4,5);
      PaintTerrRect(x-4,y-2,9,5);
      PaintTerrLine(x-3,y-3,7);
      PaintTerrLine(x-3,y+3,7);
      PaintTerrLine(x-2,y-4,5);
      PaintTerrLine(x-2,y+4,5);
      end;
    7: begin
      PaintClientRect(x-6,y-2,13,5);
      PaintClientRect(x-5,y-4,11,2);
      PaintClientRect(x-5,y+3,11,2);
      PaintClientLine(x-4,y-5,9);
      PaintClientLine(x-4,y+5,9);
      PaintClientLine(x-2,y-6,5);
      PaintClientLine(x-2,y+6,5);
      PaintTerrRect(x-6,y-2,13,5);
      PaintTerrRect(x-5,y-4,11,2);
      PaintTerrRect(x-5,y+3,11,2);
      PaintTerrLine(x-4,y-5,9);
      PaintTerrLine(x-4,y+5,9);
      PaintTerrLine(x-2,y-6,5);
      PaintTerrLine(x-2,y+6,5);
      end;
    8: begin
      mpBrushPar.size:=21;
      PaintClientRect (x-10, y-10, 21, 21);
      PaintTerrRect (x-10, y-10, 21, 21);
      end;
  end;
end;

procedure PaintTerrTrack; {------------------------------------}
var xpos, ypos, xdiff, ydiff, anz,i: integer;
begin
  if ((abs(diffx) > mpTerrDist) or (abs(diffy) > mpTerrDist)) then begin
    xdiff := x2 - x1;
    ydiff := y2 - y1;
    anz := abs (xdiff) div mpTerrDist;
    if abs (ydiff div mpTerrDist) > anz then anz := abs (ydiff div mpTerrDist);
    for i := 1 to anz do begin
      xpos := round (x1 + i * xdiff / anz);
      ypos := round (y1 + i * ydiff / anz);
      PaintTerrSpot (xpos, ypos);
    end;
  end else PaintTerrSpot (x2, y2);
end;


procedure PaintBrushTrack; {----------------------------}
var xpos,ypos,xdiff,ydiff, anz,i: integer;
begin
  if ((abs(diffx) > mpElevDist) or (abs(diffy) > mpElevDist)) then begin
    xdiff := x2 - x1;
    ydiff := y2 - y1;
    anz := abs (xdiff) div mpElevDist;
    if abs (ydiff div mpElevDist) > anz then anz := abs (ydiff div mpElevDist);
    for i := 1 to anz do begin
      xpos := round (x1 + i * xdiff / anz);
      ypos := round (y1 + i * ydiff / anz);
      PaintBrushSpot (xpos, ypos);
    end;
  end else PaintBrushSpot (x2, y2);
end;

procedure PaintClientPixel; {-------------------------------}
var xx, yy:integer;
begin
  if mpScale = 1 then begin
    mpClientCanvas.Pixels[x,y] := col;
  end else begin
    mpClientCanvas.brush.color := col;
    mpClientCanvas.pen.color := col;
    xx := x * mpScale;
    yy := y * mpScale;
    mpClientCanvas.rectangle (xx, yy, xx + mpScale, yy + mpScale);
  end;
end;

procedure PaintClientLine; {-------------------------------}
var xx, yy: integer;
begin
  if mpScale = 1 then begin
    mpClientCanvas.MoveTo (x, y);
    mpClientCanvas.LineTo (x + lng, y);
  end else begin
    xx:=x * mpScale;
    yy:=y * mpscale;
    mpClientCanvas.rectangle (xx, yy, xx + mpScale * lng, yy + mpScale);
  end;
end;

procedure PaintClientRect; {--------------------------------}
var xx, yy: integer;
begin
  if mpScale = 1 then begin
    mpClientCanvas.Rectangle (x, y, x + width, y + height);
  end else begin
    xx := x * mpScale;
    yy := y * mpScale;
    mpClientCanvas.Rectangle (xx, yy, xx + mpScale * width, yy + mpScale * height);
  end;
end;

procedure PaintArrRect; {----------------------------}
var dx,dy: integer;
begin
  if x1 < 0 then x1 := 0;
  if x2 >= iiWidth then x2 := iiWidth-1;
  if y1 < 0 then y1 := 0;
  if y2>=iiHeight then y2:=iiHeight-1;
  if (x2>=x1) and (y2>=y1) then begin
    case mpMaskMode of
      0: for dy:=y1 to y2 do for dx:=x1 to x2 do EArr[dy,dx] := mpElevGrey;
      1: for dy:=y1 to y2 do for dx:=x1 to x2 do MaskArr[dy,dx]:=1;
      2: for dy:=y1 to y2 do for dx:=x1 to x2 do MaskArr[dy,dx]:=0;
    end;
  end;
end;

procedure PaintArrLine; {----------------------------}

var dx:integer;
begin
  if x1<0 then x1:=0;
  if x2>=iiWidth then x2:=iiWidth-1;
  if y1<0 then y1:=0;
  if y1>=iiHeight then y1:=iiHeight-1;
  if x2>x1 then begin
    case mpMaskMode of
      0: for dx:=x1 to x2 do EArr[y1,dx]:=mpElevGrey;
      1: for dx:=x1 to x2 do MaskArr[y1,dx]:=1;
      2: for dx:=x1 to x2 do MaskArr[y1,dx]:=0;
    end;
  end;
end;

procedure GenerateBrush; {-------------------------------------}
var halfsize, xcount, ycount, xi, yi, i, j: integer;
    max, x, y, dist, dist1: double;
    val: byte;
    withzero: boolean;
begin
  if size < 2 then size := 2;
  if size > 41 then size := 41;

  if not soft then begin // hard brush
    for i := 0 to 41 do for j := 0 to 41 do ElevBrushArr[i,j] := 0;
    halfsize := size div 2;
    withzero := (size mod 2) <> 0;
    max := halfsize - 0.5;
    ycount := -1;
    for yi := -halfsize to halfsize do begin
      if (yi <> 0) or withzero then begin
        inc (ycount);
        xcount := -1;
        for xi := -halfsize to halfsize do begin
          if (xi <> 0) or withzero then begin
            inc(xcount);
            if abs(yi) <> 0 then y := abs(yi) - 0.5 else y := 0;
            if abs(xi) <> 0 then x := abs(xi) - 0.5 else x := 0;
            dist := sqrt (x * x + y * y);
            if dist > halfsize then val := 0 else val := 100;
            ElevBrushArr [ycount, xcount] := val;
          end;
        end;
      end;
    end;

  end else begin // soft brush
    for i := 0 to 41 do for j := 0 to 41 do ElevBrushArr[i,j] := 0;
    halfsize := size div 2;
    withzero := (size mod 2) <> 0;
    max := halfsize - 0.5;
    ycount := -1;
    for yi := -halfsize to halfsize do begin
      if (yi <> 0) or withzero then begin
        inc (ycount);
        xcount := -1;
        for xi := -halfsize to halfsize do begin
          if (xi <> 0) or withzero then begin
            inc (xcount);
            if abs (yi) <> 0 then y := abs(yi) - 0.5 else y := 0;
            if abs (xi) <> 0 then x := abs(xi) - 0.5 else x := 0;
            dist := sqrt (x * x + y * y);
            dist1 := intens - dist * intens / halfsize;
            if dist1 < 0 then dist1 := 0;
            val := round (dist1);
            ElevBrushArr [ycount, xcount] := val;
          end;
        end;
      end;
    end;
  end;

  mpBrushPar.size := size;
  mpBrushPar.intens := intens;
  mpBrushPar.soft := soft;

  mpElevDist := mpBrushPar.size div 6;
  if mpElevDist < 1 then mpElevDist := 1;
  if mpElevDist > 3 then mpElevDist := 3;
  GenerateBrightBrush;
end;

procedure GenerateBrightBrush; {-------------------------------}
var i, j: integer;
begin
  for i := 0 to mpBrushPar.size-1 do for j := 0 to mpBrushPar.size-1 do
  if mpElevBrush < 9 then begin
    BrightBrushArr[i,j] := round (ElevBrushArr[i,j] * abs (mpBright) / 100);
  end else begin
    BrightBrushArr[i,j] := round (ElevBrushArr[i,j] * abs (mpBright) / 200);
  end;
end;

procedure PaintBrushSpot; {------------------------------------}
var val1, val2, newgrey, dx, dy: integer;
    xfirst, yfirst, xpos, ypos: integer;
    col: TColor;
    oldgrey: byte;

begin
  if iiOpen = false then exit;

  // set client-canvas color
  case mpMaskMode of
    0: col := cCol (mpElevGrey, mpElevGrey, mpElevGrey);
    1: col := iiMaskCol;
    2: col := iiUnmaskCol;
  end;
  mpClientCanvas.Brush.Color := col;
  mpClientCanvas.Pen.Color := col;

  case mpElevBrush of
    1: begin
      PaintClientPixel (x, y, col);
      PaintArrRect (x, y, x, y);
      end;
    2: begin
      PaintClientRect (x-1, y-1, 2, 2);
      PaintArrRect (x-1, y-1, x, y);
      end;
    3: begin
      PaintClientRect (x-1, y-1, 3, 3);
      PaintArrRect (x-1, y-1, x+1, y+1);
      end;
    4: begin
      PaintClientRect (x-2, y-1, 5, 3);
      PaintClientLine (x-1, y-2, 3);
      PaintClientLine (x-1, y+2, 3);
      PaintArrRect (x-2, y-1, x+2, y+1);
      PaintArrLine (x-1, y-2, x+1);
      PaintArrLine (x-1, y+2, x+1);
      end;
    5: begin
      PaintClientRect(x-3,y-1,7,3);
      PaintClientLine(x-2,y-2,5);
      PaintClientLine(x-2,y+2,5);
      PaintClientLine(x-1,y-3,3);
      PaintClientLine(x-1,y+3,3);
      PaintArrRect(x-3,y-1,x+3,y+1);
      PaintArrLine(x-2,y-2,x+2);
      PaintArrLine(x-2,y+2,x+2);
      PaintArrLine(x-1,y-3,x+1);
      PaintArrLine(x-1,y+3,x+1);
      end;
    6: begin
      PaintClientRect(x-4,y-2,9,5);
      PaintClientLine(x-3,y-3,7);
      PaintClientLine(x-3,y+3,7);
      PaintClientLine(x-2,y-4,5);
      PaintClientLine(x-2,y+4,5);
      PaintArrRect(x-4,y-2,x+4,y+2);
      PaintArrLine(x-3,y-3,x+3);
      PaintArrLine(x-3,y+3,x+3);
      PaintArrLine(x-2,y-4,x+2);
      PaintArrLine(x-2,y+4,x+2);
      end;
    7: begin
      PaintClientRect(x-6,y-2,13,5);
      PaintClientRect(x-5,y-4,11,2);
      PaintClientRect(x-5,y+3,11,2);
      PaintClientLine(x-4,y-5,9);
      PaintClientLine(x-4,y+5,9);
      PaintClientLine(x-2,y-6,5);
      PaintClientLine(x-2,y+6,5);
      PaintArrRect(x-6,y-2,x+6,y+2);
      PaintArrRect(x-5,y-4,x+5,y-3);
      PaintArrRect(x-5,y+3,x+5,y+4);
      PaintArrLine(x-4,y-5,x+4);
      PaintArrLine(x-4,y+5,x+4);
      PaintArrLine(x-2,y-6,x+2);
      PaintArrLine(x-2,y+6,x+2);
      end;
    8: begin
      PaintClientRect (x-10, y-10, 21, 21);
      PaintArrRect (x-10, y-10, x+10, y+10);
      end;
    9..16: if mpMaskMode < 1 then begin
      xfirst := x - mpBrushPar.size div 2;
      yfirst := y - mpBrushPar.size div 2;
      for dy := 0 to mpBrushPar.size-1 do begin
        ypos := yfirst + dy;
        for dx :=0 to mpBrushPar.size - 1 do begin
          xpos := xfirst + dx;
          if (xpos >= 0) and (xpos < EMap.Width) and (ypos >= 0) and (ypos < EMap.Height) then begin
            val1 := ElevBrushArr[dx,dy];
            if val1 > 100 then val1 := 100;
            val2 := 100 - val1;
            oldgrey := EArr[ypos, xpos];
            newgrey := (val2 * oldgrey + val1 * mpElevGrey) div 100;
            EArr[ypos,xpos] := newgrey;
            col := cCol (newgrey, newgrey, newgrey);
            PaintClientPixel (xpos, ypos, col);
          end;
        end;
      end;
      end;
  end;
end;

procedure PaintBlurSpot; {----------------------------}
var dx, dy, grey, xfirst, yfirst, ypos, xpos, intens: integer;
    col: TColor;
begin
  xfirst := x - mpBrushPar.size div 2;
  yfirst := y - mpBrushPar.size div 2;
  for dy := 0 to mpBrushPar.size-1 do begin
    ypos := yfirst + dy;
    for dx :=0 to mpBrushPar.size - 1 do begin
      xpos := xfirst + dx;
      if mpElevBrush < 9 then intens := round (ElevBrushArr[dx,dy] * mpBlur / 255)
        else intens := round (ElevBrushArr[dx,dy] * mpBlur / 127);
      if intens > 10 then intens := 10;
      BlurArr[dx,dy] := BlurDot (xpos, ypos, intens);
    end;
  end;
  for dy := 0 to mpBrushPar.size-1 do begin
    ypos := yfirst + dy;
    for dx :=0 to mpBrushPar.size - 1 do begin
      xpos := xfirst + dx;
      if (xpos >= 0) and (xpos < iiWidth) and (ypos >= 0) and (ypos < iiHeight) then begin
        grey := BlurArr[dx,dy];
        EArr[ypos,xpos] := grey;
        PaintClientPixel (xpos, ypos, cCol (grey,grey,grey));
      end;
    end;
  end;
end;



procedure M_RecallMMap;
begin
  if not iiOpen then exit;
  MMap.canvas.draw (0, 0, MMap2);
end;

procedure M_RecallMMapRect; {---------------------------}
begin
  MMap.Canvas.Copyrect (rr, MMap2.Canvas, rr);
end;

procedure M_StoreMMap;
begin
  if not iiOpen then exit;
  MMap2.height := MMap.height;
  MMap2.width := MMap.width;
  MMap2.canvas.draw (0, 0, MMap);
end;

function BlurDot; {-----------------------------------}
var dx,dy,sum:integer;
begin
  if (x > 0) and (x < iiWidth-1) and (y > 0) and (y < iiHeight-1) then begin
    sum:=0;
    for dx:=x-1 to x+1 do for dy:=y-1 to y+1 do sum:=sum+EArr[dy,dx];
    Result:=round ((sum/9 * (intens) + Earr[y,x] * (10 - intens)) / 10);
  end else begin
    if (x < 0) or (x >= iiWidth) or (y < 0) or (y >= iiHeight) then begin
      Result := 0;
    end else begin
      Result:= EArr[y,x];
    end;
  end;
end;

procedure PaintBlurTrack; {----------------------------}
var xpos, ypos, xdiff, ydiff, anz, i: integer;
begin
  if ((abs (diffx) > 3) or (abs (diffy) > 3)) then begin
    xdiff := x2 - x1;
    ydiff := y2 - y1;
    anz := abs (xdiff) div mpElevDist;
    if abs (ydiff div mpElevDist) > anz then anz := abs (ydiff div mpElevDist);
    for i:=1 to anz do begin
      xpos := round (x1 + i * xdiff / anz);
      ypos := round (y1 + i * ydiff / anz);
      PaintBlurSpot (xpos, ypos);
    end;
  end else PaintBlurSpot (x2, y2);
end;

procedure PaintBrightSpot; {--------------------------}
var xfirst, yfirst, dy, dx, ypos, xpos : integer;
    diffgrey, oldgrey, addgrey, donegrey, newgrey: integer;
    col: TColor;
begin
  xfirst := x - mpBrushPar.size div 2;
  yfirst := y - mpBrushPar.size div 2;
  for dy := 0 to mpBrushPar.size-1 do begin
    ypos := yfirst + dy;
    for dx :=0 to mpBrushPar.size-1 do begin
      xpos := xfirst + dx;
      if (xpos >= 0) and (xpos < EMap.Width) and (ypos >= 0) and (ypos < EMap.Height) then begin
        addgrey := BrightBrushArr [dx, dy];
        oldgrey := EArr [ypos, xpos];
        donegrey := LockArr [ypos, xpos];

        if mpPaintLock then begin
          if addgrey > donegrey then begin
            diffgrey := addgrey - donegrey;

            if mpBright > 0 then begin // lighten
              newgrey := oldgrey + diffgrey;
              if newgrey > 255 then newgrey := 255;
            end else begin  // darken
              newgrey := oldgrey - diffgrey;
              if newgrey < 0 then newgrey := 0;
            end;

            LockArr [ypos, xpos] := LockArr [ypos, xpos] + diffgrey;
            EArr [ypos, xpos] := newgrey;
            col := cCol (newgrey, newgrey, newgrey);
            PaintClientPixel (xpos, ypos, col);
          end;
        end else begin
          if mpBright > 0 then begin // lighten
            newgrey := Oldgrey + addgrey;
            if newgrey > 255 then newgrey := 255;
          end else begin // darken
            newgrey := Oldgrey - addgrey;
            if newgrey < 0 then newgrey := 0;
          end;
          EArr [ypos, xpos] := newgrey;
          col := cCol (newgrey, newgrey, newgrey);
          PaintClientPixel (xpos, ypos, col);
        end;
      end;
    end;
  end;
end;

procedure PaintBrightTrack; {----------------------------}
var xpos, ypos, xdiff, ydiff, anz, i: integer;
begin
  if (abs (diffx) > 1) or (abs (diffy) > 1) then begin
    xdiff := x2 - x1;
    ydiff := y2 - y1;
    anz := abs (xdiff) div mpElevDist;
    if abs (ydiff div mpElevDist) > anz then anz := abs (ydiff div mpElevDist);
    for i:=1 to anz do begin
      xpos := round (x1 + i * xdiff / anz);
      ypos := round (y1 + i * ydiff / anz);
      PaintBrightSpot (xpos, ypos);
    end;
  end else PaintBrightSpot (x2, y2);
end;

procedure PaintSprayTrack; {----------------------------}
var xpos, ypos, xdiff, ydiff, anz, i: integer;
begin
  if (abs (diffx) > 1) or (abs (diffy) > 1) then begin
    xdiff := x2 - x1;
    ydiff := y2 - y1;
    anz := abs (xdiff) div mpElevDist;
    if abs (ydiff div mpElevDist) > anz then anz := abs (ydiff div mpElevDist);
    for i:=1 to anz do begin
      xpos := round (x1 + i * xdiff / anz);
      ypos := round (y1 + i * ydiff / anz);
      PaintSpraySpot (xpos, ypos);
    end;
  end else PaintSpraySpot (x2, y2);
end;


end.
