unit UStandard;

{ Changing for Windows:
- change SEP to '\'
- commend out linux functions
}

{----------------} interface {---------------------------------}

uses
  QGraphics, Types, SysUtils, Libc, Classes, Math, QDialogs;

const
  SEP: char = '/';
  OperatingSystem: String = 'Linux';

{--- Linux and Windows ---}
function  HomeDir: String;
procedure CpFile (srcfile, destdir: string);
function  CpDir  (srcdir, destcontainerdir: String): boolean;
procedure XCopy  (srcdir, destcontainerdir: String);

{--- only Linux ---}
(*function  CopyFileBkgnd (source, dest: String): boolean;
function  CopyDirBkgnd  (source, dest: String): boolean;
function  DelFile (path: String): boolean;
function  RestoreDir (source, dest:String): boolean;
function  DelDir (path:String): boolean;
function  StartProg (prog, param:String): boolean;*)


{**************************************************************}
function  DelPart (s:String; part:String): String;
function  ChangePart (s: String; part1, part2: String): String;

procedure RotateIntPoint (angle: double; dpx, dpy, oldx, oldy: integer; var newx, newy:integer);
procedure RotatePoint (angle, oldx, oldy: double; var newx, newy: double);

function  dsin (angle:double): double;
function  dcos (angle:double): double;
function  dtan (angle:double): double;
function  darcsin (val:double): double;
function  darccos (val:double): double;
function  darctan (val:double): double;

function  cCol (r,g,b: integer): TColor;
function  cRed (col: TColor): integer;
function  cGreen (col: TColor): integer;
function  cBlue (col:TColor): integer;
function  MixCol (col1,col2,val2:TColor): TColor;

function  XRandom (min,max:integer): integer;

function  BareStr       (const s:String): string;
function  Int_Str       (i:longint; ZeroToSpace:boolean): string;
function  Str_Int       (const s:string; def:longint): longint;
function  Str_Float     (const s:String; def: Double): Double;
function  Float_Str     (const v:double; kom:integer): String;

function  FormRR        (rr:TRect):TRect;
function  MoveRR        (const r:TRect; XOffs,YOffs:integer): TRect;
function  ExpandRR      (const r:TRect; offs:integer): TRect;

procedure PrintFrame    (c:TCanvas; li,ob,re,un:integer);
procedure PrintRFrame   (c:TCanvas; r:TRect);
procedure PrintHLin     (c:TCanvas; x,y,lng:integer);
procedure PrintVLin     (c:TCanvas; x,y,lng:integer);

{-------   Parser ----------------------------------------------}
function   ExtParsStr   (const line,item:String): String;
function   ParsStr      (const line,item:String): String;
function   ParsInt      (const line,item:String; def:integer): integer;
function   ParsFloat    (const line,item:String; def:double): double;
function   ParsRect     (const line:String): TRect;
function   ParsStr2     (const line,item:String): String;
function   ParsInt2     (const line,item:String; def:integer): integer;
function   ParsFloat2   (const line,item:String; def:double): double;
procedure  SetParsStr   (var line:String; item,new:String);
procedure  SetParsStr2  (var line:String; item,new:String);
procedure  SetParsInt   (var line:String; item:String; new:integer);
procedure  SetParsFloat (var line:String; item:String; new:double; kom:integer);
procedure  SetParsRect  (var line:String; r:TRect);
procedure  SetParsRect2 (var line:String; r:TRect);

procedure  SeparateParams  (var line: String; item: String);
function   SeparateTCLPath (const path: String; var a, b: String): boolean;
function   SeparateOSPath  (const path: String; var a, b: String): boolean;


var
  ProgDir: String;

{----------------} implementation {----------------------------}

(*
{=================== Windows ==================================}

function HomeDir; {--------------------------------------------}
  var pidl: PItemIDList;
      path: PChar;
begin
  if SUCCEEDED(SHGetSpecialFolderLocation (0, CSIDL_PERSONAL, pidl)) then begin
    path := StrAlloc (max_path);
    SHGetPathFromIDList (pidl, path);
    Result := String (path);
    if Result [length (Result)] <> '\' then Result := Result + '\';
  end;
end;

procedure CpFile; {--------------------------------------------}
begin
  CopyFile (PChar(srcfile), PChar(destdir), true);
end;

function CpDir; {----------------------------------------------}
var
  fos: TSHFileOpStruct;
begin
  ZeroMemory(@fos, SizeOf (fos));
  with fos do
  begin
    wFunc  := FO_COPY;
    fFlags := FOF_FILESONLY;
    pFrom  := PChar (srcdir + #0);
    pTo    := PChar (destcontainerdir)
  end;
  Result := (0 = ShFileOperation (fos));
end;

procedure XCopy; {---------------------------------------------}
var parstr: String;
begin
  parstr := '"' + srcdir + '" "' + destcontainerdir + '" /e';
  ShellExecute (0, 'open', 'xcopy', PChar(parstr), '', SW_SHOWNORMAL);
end;

{========================== End Windows =======================}
*)


{======================== Linux ===============================}
function HomeDir; {--------------------------------------------}
begin
  Result := GetEnvironmentVariable ('HOME');
end;

procedure CpFile; {--------------------------------------------}
begin
  Libc.System (PChar ('cp ' + srcfile + ' ' + destdir));
end;

function CpDir; {----------------------------------------------}
begin
  if DirectoryExists (srcdir) then begin
    Libc.System (PChar ('cp -R ' + srcdir + ' ' + destcontainerdir));
  end;
end;

procedure XCopy; {---------------------------------------------}
begin
  Libc.System (PChar ('cp -R ' + srcdir + '/* ' + destcontainerdir));
end;

(*
function CopyFileBkgnd; {--------------------------------------}
begin
  if FileExists (source) then begin
    Libc.System (PChar ('cp ' + source + ' ' + dest + ' &'));
    Result := true;
  end else Result := false;
end;

function CopyDirBkgnd; {---------------------------------------}
// source must exist - dest must not exist
begin
  if (DirectoryExists (source))
  and (not DirectoryExists (dest)) then begin
    Libc.System (PChar ('cp -r ' + source + ' ' + dest + ' &'));
    Result :=  true;
  end else Result := false;
end;

function DelFile; {--------------------------------------------}
begin
  if FileExists (path) then begin
    Libc.System (PChar ('rm ' + path + ' &'));
    Result := true;
  end else Result := false;
end;

function RestoreDir; {-----------------------------------------}
// source and dest must exist - dest will be deleted first
begin
  // check if restore script (linux) exists in ProgDir
  if not FileExists (Progdir + 'restoredir') then begin
    Result := false;
    exit;
  end else begin
    if DirectoryExists (source) and DirectoryExists (dest) then begin
      Libc.System (PChar (ProgDir + 'restoredir ' + source + ' ' + dest + ' &'));
      Result := true;
    end else Result := false;
  end;
end;

function DelDir; {---------------------------------------------}
begin
  if DirectoryExists (path) then begin
    Libc.System (PChar ('rm -rf ' + path + ' &'));
    Result := true;
  end else Result := false;
end;

function StartProg; {------------------------------------------}
begin
  if Libc.System(PChar(prog+' '+param+' &')) = -1 then
  ShowMessage('error');
end;
{================== End Linux ==============================}
*)

function DelPart; {---------------------------------------------}
var ss: String;
    po: integer;
begin
  ss := s;
  po := pos (part, ss);
  if po <> 0 then delete (ss, po, length(part));
  Result := ss;
end;

function ChangePart; {-----------------------------------------}
var po: integer;
    ss: String;
begin
  ss := s;
  po := pos (part1, ss);
  if po <> 0 then begin
    Delete (ss, po, length(part1));
    Insert (part2, ss, po);
  end;
  Result := ss;
end;

procedure RotateIntPoint; {------------------------------------}
var x,y: double;
begin
  x := (oldx - dpx) * cos(angle) - (oldy - dpy) * sin(angle) + dpx;
  y := (oldx - dpx) * sin(angle) + (oldy - dpy) * cos(angle) + dpy;
  newx := Round(x);
  newy := Round(y);
end;

procedure RotatePoint; {---------------------------------------}
begin
  newx := oldx * cos(angle) - oldy * sin(angle);
  newy := oldx * sin(angle) + oldy * cos(angle);
end;

function dsin;
begin
  Result:=sin(angle * pi / 180);
end;

function dcos;
begin
  Result:=cos(angle * pi / 180);
end;

function dtan;
begin
  Result:=tan(angle * pi / 180);
end;

function darcsin;
begin
  Result:=arcsin(val) * 180 / pi;
end;

function darccos;
begin
  Result:=arccos(val) * 180 / pi;
end;

function darctan;
begin
  Result:=arctan(val) * 180 / pi;
end;

function cCol; {-----------------------------------------------}
begin
  Result := r + g shl 8 + b shl 16;
end;

function cRed; {-----------------------------------------------}
begin
  Result := col and $000000FF;
end;

function cGreen; {---------------------------------------------}
begin
  Result := (col and $0000FF00) shr 8;
end;

function cBlue;
begin
  Result := (col and $00FF0000) shr 16;
end;

function MixCol; {---------------------------------------------}
var r1,g1,b1,r2,g2,b2,r,g,b,val1: integer;
begin
  if val2>100 then val2:=100;
  val1:=100 - val2;
  r1:=cRed(col1);
  g1:=cGreen(col1);
  b1:=cBlue(col1);
  r2:=cRed(col2);
  g2:=cGreen(col2);
  b2:=cBlue(col2);
  r:=(val1*r1 + val2*r2) div 100;
  g:=(val1*g1 + val2*g2) div 100;
  b:=(val1*b1 + val2*b2) div 100;
  Result:=cCol(r,g,b);
end;

function XRandom; {--------------------------------------------}
begin
  Result := Random (max - min + 1) + min;
end;

function BareStr; {--------------------------------------------}
var str1: String;
begin
  str1 := s;
  while (length (str1) > 0) and (str1[1] in [#32, #9]) do delete (str1, 1, 1);
  while (length (str1) > 0) and (str1[length(str1)] = #32) do delete (str1, length(str1), 1);
  Result := str1;
end;

function Int_Str; {--------------------------------------------}
var ss:string;
begin
  if ZeroToSpace and (i=0) then Result:='' else begin
    str(i,ss);
    Result:=ss;
  end;
end;

function Str_Int; {--------------------------------------------}
var err,wert:integer;
begin
  if BareStr(s)='' then begin Result:=def; exit end;
  val(s,wert,err);
  if err<>0 then Result:=def else Result:=wert;
end;

function Str_Float; {------------------------------------------}
var err:integer;
    wert:double;
begin
  if BareStr(s)='' then begin Result:=def; exit end;
  val(s,wert,err);
  if err<>0 then Result:=def else Result:=wert;
end;

function Float_Str; {------------------------------------------}
begin
  Result:=FloatToStrF (v,ffFixed,12,kom);
end;

function FormRR; {---------------------------------------------}
var fr:TRect;
begin
  fr:=rr;
  if rr.left>rr.right then begin
    fr.left:=rr.right; fr.right:=rr.left;
  end;
  if rr.top>rr.bottom then begin
    fr.top:=rr.bottom; fr.bottom:=rr.top;
  end;
  Result:=fr;
end;

function MoveRR; {---------------------------------------------}
var rr:TRect;
begin
  rr.left:=r.left+xoffs;
  rr.right:=r.right+xoffs;
  rr.top:=r.top+yoffs;
  rr.bottom:=r.bottom+yoffs;
  Result:=rr;
end;

function ExpandRR; {-------------------------------------------}
var rr:TRect;
begin
  rr.left:=r.left-offs;
  rr.right:=r.right+offs;
  rr.top:=r.top-offs;
  rr.bottom:=r.bottom+offs;
  Result:=rr;
end;

procedure PrintFrame; {----------------------------------------}
var rr:TRect;
begin
  with rr do begin
    left:=li; top:=ob; right:=re; bottom:=un;
  end;
  PrintRFrame (c,rr);
end;

procedure PrintRFrame; {---------------------------------------}
var fr:TRect;
begin
  fr:=FormRR(r);
  c.MoveTo(fr.left,fr.top);
  c.LineTo(fr.right,fr.Top);
  c.LineTo(fr.right,fr.bottom);
  c.LineTo(fr.left,fr.bottom);
  c.LineTo(fr.left,fr.top);
end;

procedure PrintHLin; {-----------------------------------------}
begin
  c.MoveTo(x,y);
  c.LineTo(x+lng,y);
end;

procedure PrintVLin; {-----------------------------------------}
begin
  c.MoveTo(x,y);
  c.LineTo(x,y+lng);
end;

{--------- Parser ---------------------------------------------}

function ParsStr (const line,item:String): String;
var ss,it:string;
    anf,ende:integer;
begin
  if (line='') or (item='') then begin Result:=''; exit end;
  ss:=line;
  it:='['+item+']';
  anf:=pos(it,ss);
  if anf=0 then begin Result:=''; exit end;
  ss:=ss+'[';
  inc(anf,length(it));
  ende:=anf-1;
  repeat inc(ende) until ss[ende]='[';
  Result:=trim(copy(ss,anf,ende-anf));
end;

function ExtParsStr; {--------------------------------------------}
var ss,sss,ssss:String;
    anz,i:integer;
begin
  ss:=ParsStr(line,item);
  if (ss<>'') and (ss[1]='#') then begin
    anz:=ParsInt2(ss,'anz',0);
    sss:='';
    if anz>0 then for i:=1 to anz do begin
      ssss:=ParsStr2(ss,IntToStr(i));
      sss:=sss+ParsStr(line,ssss);
    end;
    Result:=sss;
  end else Result:=ss;
end;

function ParsInt; {-------------------------------------------}
var zwi:string;
begin
  zwi:=ParsStr(line,item);
  if zwi='' then Result:=def else Result:=Str_Int(zwi,def);
end;

function ParsFloat; {--------------------------------------------}
var zwi:string;
begin
  zwi:=ParsStr(line,item);
  if zwi='' then Result:=def else Result:=Str_Float(zwi,def);
end;

function ParsRect; {-------------------------------------------}
var rr:TRect;
begin
  rr.top:=ParsInt(line,'ob',0);
  rr.left:=ParsInt(line,'li',0);
  rr.right:=ParsInt(line,'re',0);
  rr.bottom:=ParsInt(line,'un',0);
  Result:=rr;
end;

function ParsStr2; {-------------------------------------------}
var ss,it:string;
    anf,ende:integer;
begin
  if (line='') or (item='') then begin Result:=''; exit end;
  ss:=line;
  it:='{'+item+'}';
  anf:=pos(it,ss);
  if anf=0 then begin Result:=''; exit end;
  ss:=ss+'{';
  inc(anf,length(it));
  ende:=anf-1;
  repeat inc(ende) until ss[ende]='{';
  Result:=trim(copy(ss,anf,ende-anf));
end;

function ParsInt2; {-------------------------------------------}
var zwi:string;
begin
  zwi:=ParsStr2(line,item);
  if zwi='' then Result:=def else Result:=Str_Int(zwi,def);
end;

function ParsFloat2; {-------------------------------------------}
var zwi:string;
begin
  zwi:=ParsStr2(line,item);
  if zwi='' then Result:=def else Result:=Str_Float(zwi,def);
end;

procedure SetParsStr; {----------------------------------------}
var anf,ende:integer;
    it:String;
begin
  it:='['+item+']';
  anf:=pos(it,line);
  if anf=0 then begin
    line:=line+it+new;
  end else begin
    line:=line+'[';
    inc(anf,length(it));
    ende:=anf-1;
    repeat inc(ende) until line[ende]='[';
    Delete(line,anf,ende-anf);
    Insert(new,line,anf);
    Delete(line,length(line),1);
  end;
end;

procedure SetParsStr2; {---------------------------------------}
var anf,ende:integer;
    it:String;
begin
  it:='{'+item+'}';
  anf:=pos(it,line);
  if anf=0 then begin
    line:=line+it+new;
  end else begin
    line:=line+'{';
    inc(anf,length(it));
    ende:=anf-1;
    repeat inc(ende) until line[ende]='{';
    Delete(line,anf,ende-anf);
    Insert(new,line,anf);
    Delete(line,length(line),1);
  end;
end;

procedure SetParsInt; {----------------------------------------}
begin
  SetParsStr(line,item,IntToStr(new));
end;

procedure SetParsFloat; {--------------------------------------}
begin
  SetParsStr(line,item,Float_Str(new,kom));
end;

procedure SetParsRect; {---------------------------------------}
begin
  SetParsStr(line,'ob',IntToStr(r.top));
  SetParsStr(line,'li',IntToStr(r.left));
  SetParsStr(line,'re',IntToStr(r.right));
  SetParsStr(line,'un',IntToStr(r.bottom));
end;

procedure SetParsRect2; {--------------------------------------}
begin
  SetParsStr2(line,'ob',IntToStr(r.top));
  SetParsStr2(line,'li',IntToStr(r.left));
  SetParsStr2(line,'re',IntToStr(r.right));
  SetParsStr2(line,'un',IntToStr(r.bottom));
end;

procedure SeparateParams; {-------------------------------------}
var anz,po:integer;
    a: String;
begin
  a := ' ' + BareStr (ParsStr (line, item));
  if a <> ' ' then begin
     anz := 0;
     repeat
       po := pos(' ', a);
       if po > 0 then begin
         inc (anz);
         delete (a, po, 1);
         insert ('{' + IntToStr(anz) + '}', a, po);
       end;
     until po = 0;
     SetParsStr (line, item, a);
  end;
end;


function SeparateTCLPath; {---------------------------------------}
var po: integer;
begin
  if pos ('/', path) = 0 then begin
    a := path;
    b := '';
    Result := false;
  end else begin
    po := length (path) + 1;
    repeat dec(po) until path[po] = '/';
    a := copy (path, 1, po-1);
    b := copy (path, po+1, 100);
  end;
end;


function SeparateOSPath; {---------------------------------------}

var po: integer;
begin
  if pos (SEP, path) = 0 then begin
    a := path;
    b := '';
    Result := false;
  end else begin
    po := length (path) + 1;
    repeat dec(po) until path[po] = SEP;
    a := copy (path, 1, po-1);
    b := copy (path, po+1, 100);
  end;
end;


{---------------- initialization ------------------------------}

begin
  Randomize;
  ProgDir := ExtractFilePath (ParamStr(0));
end.
