unit UTuxbase;

interface

uses
  QGraphics, Types, SysUtils, Libc, Classes, Math, QDialogs, UStandard,
  QExtCtrls;

const
  MaxHeight = 2500;   // max size of bitmaps
  MaxWidth  = 200;
  MaxElevStore  = 10;
  MaxTerrStore  = 3;

  etArr     = 1;  // elevation edit tools
  etBrush   = 2;
  etBlur    = 3;
  etDark    = 5;
  etPip     = 6;
  etMask    = 7;
  etSpray   = 8;


  {--- edit modes ---}
  emNone   = 0;
  emCourse = 1;  // editing course params
  emElev   = 2;  // editing elevation
  emTerr   = 3;  // editing terrain
  emObj    = 4;  // editing objects
  emTarg   = 5;  // editing am_targets
  emIntern = 6;  // editing environment
  emScript = 7;  // editing random scripts
  

  {--- object tools ---}
  otNone   = 0;
  otEdit   = 1;
  otAdd    = 2;
  otMulti  = 3;
  otDel    = 4;
  otDel2   = 5;

  coDarkBlue: TColor = $00783C1E;
  coLightBlue: TColor = $00FFB478;

type
  TMapArr = array[0..MaxHeight, 0..MaxWidth] of byte;
  TBrushArr = array[0..41, 0..41] of byte;
  TBrushPar = record
    size: integer;
    intens: integer;
    soft: boolean;
  end;


var
  {--- user settings ---}
  iiShowOutlines: boolean;  // for config file
  iiOutlineCol: TColor;
  iiRefreshSize: integer;
  iiMaskCol: TColor;
  iiUnmaskCol: TColor;

  EMap: TBitmap;      // elevation-bitmap 1:1
  EArr: TMapArr;      // elevation grey-array 1:1
  TMap: TBitmap;      // terrain-bitmap 1:1
  MMap: TBitmap;      // mixed bitmap 1:FScale
  MMap2: TBitmap;     // for local use without objects

  TerrStore: array [1..MaxTerrStore] of TBitmap;   // for terrain undo 1:1
  TerrPoint: array [1..MaxTerrStore] of integer;
  TerrCount: integer;
  ElevStore: array [1..MaxElevStore] of TMapArr;   // for undo
  ElevPoint: array [1..MaxElevStore] of integer;   // for undo
  ElevCount: integer;                              // for undo

  BlurArr: TBrushArr;        // contains the avarage values of blurred dots
  ElevBrushArr: TBrushArr;
  BrightBrushArr: TBrushArr;
  MaskArr: TMapArr;
  LockArr: TMapArr;

  OArr: TStringlist;    // Objekte auf dem Kurs
  OList: TStringlist;   // Komplette Objektliste
  OIdx: String;         // String mit belegten X-Y-Positionen
  OTypIdx: STring;
  ONameIdx: String;

  iiTuxBin: String;      // path to tuxracer binary
  iiCourseDir: String;   // path to main course directory
  iiHomeDir: String;     // users home directory
  iiConfDir: String;     // .tuxeditor in HomeDir
  iiTrackDir: String;    // path to track directory
  iiCupDir: String;      // path to cup directory
  iiTrackName: String;
  iiCupName: String;
  iiObjResource: String;

  iiCourseDirOK: boolean;
  iiTerrlistOK: boolean;
  iiObjlistOK: boolean;
  iiOpen: boolean;

  mpScale: integer;          // Scale of client map;
  mpMaskMode: integer;       // Maskierung der Map
  mpObjBright: integer;      // brightness of object background (1..5)
  mpMix: integer;            // transparancy of terrain bitmap
  mpElevGrey: integer;       // for editing with brush etc.
  mpBackGrey: integer;
  mpElevTool: integer;       // see et-constants
  mpElevBrush: integer;
  mpBrushPar: TBrushPar;
  mpElevDist: integer;
  mpBright: integer;
  mpPaintLock: boolean;
  mpTerrCol: TColor;
  mpTerrBrush: integer;
  mpTerrBrushSize: integer;
  mpTerrDist: integer;
  mpBlur: integer;
  mpVall: integer;
  mpMount: integer;
  mpMinSpray: integer;
  mpMaxSpray: integer;
  mpBasementAngle: integer;

  {--- modes ---}
  EditMode: integer;            // see em-constants
  ObjectTool: integer;          // see ot-constants

  {--- bitmap ---}
  iiHeight: integer;            // height (pixels) of EMap
  iiWidth: integer;

  {--- course.tcl ---}
  iiCourseLength: double;
  iiCourseWidth: double;
  iiPlayLength: double;
  iiPlayWidth: double;
  iiAngle: double;
  iiScale: double;
  iiStartStr: string;
  iiCourseName: string;

  {--- edit flags ---}
  iiElevChanged: boolean;
  iiTerrChanged: boolean;
  iiObjChanged: boolean;
  iiTargChanged: boolean;
  iiEnvChanged: boolean;
  iiCourseChanged: boolean;

  FMarkNum: integer;  // fr Objektmarkierung
  FMarkStr: String;   // fr markierte Objekte
  AreaFlag: String;

function  B_LoadObjlist: boolean;
function  B_CheckStructure (panel: TPanel): boolean;
function  B_LoadConfig: boolean;
function  B_SaveConfig: boolean;
procedure B_SetMapSize;

procedure EArr_EMap;
procedure EMap_EArr;
procedure EArrRect_MMap (r: TRect);
procedure EArr_MMap;
procedure TMapRect_MMap (r: TRect);
procedure TMap_MMap;

function  B_XPos    (bmx: double): double;
function  B_XPixPos (x: double): double;  // invers to xPos
function  B_YPos    (bmy: double): double;
function  B_YPixPos (y: double): double;  // invers to yPos
function  B_ZPos    (bmz, bmy: double): double;
function  B_ZPos2   (zgrey, bmy: double): double;
function  B_ZGrey2  (zpos, bmy: double): double;

function  B_ElevFile: String;      // elev.png
function  B_TerrainFile: String;   // terrain.png
function  B_OTexturesFile: String; // object_textures.tcl
function  B_ObjectsFile: String;   // objects.tcl
function  B_CourseFile: String;
function  B_OArrFile: String;
function  B_ItemsFile: String;

procedure B_MakeOIdx;

implementation

function B_LoadObjlist; {--------------------------------------}
var i: integer;
    ss: String;
begin
  if FileExists (ProgDir + 'res' + SEP + 'objects.lst') then begin
    OList.LoadFromFile (ProgDir + 'res' + SEP + 'objects.lst');
    OTypIdx := '';
    ONameIdx := '';
    if OList.count > 0 then begin
      for i := 0 to OList.Count-1 do begin
        ss := OList[i];
        if ParsStr (ss, 'struct') = 'obj' then begin
          OTypIdx := OTypIdx + '[' + ParsStr (ss, 'type') + ']' + IntToStr(i);
          ONameIdx := ONameIdx + '[' + ParsStr (ss, 'name') + ']' + IntToStr(i);
          while pos('{',ss) > 0 do ss[pos('{', ss)] := '[';
          while pos('}',ss) > 0 do ss[pos('}', ss)] := ']';
        end;
        OList[i] := ss;
      end;
    end;
    iiObjlistOK := true;
  end else iiObjlistOK := false;
end;

function B_CheckStructure; {-----------------------------------}
begin
  if iiCourseDir = '' then iiCourseDirOK := false
  else iiCourseDirOK := FileExists (iiCourseDir + SEP + 'course_idx.tcl');
  if iiCourseDirOK then begin
    panel.visible := false;
//    ForceDirectories(iiCourseDir+SEP+'_backup');
//    ForceDirectories(iiCourseDir+SEP+'contrib');
//    U_PrepareContrib;
  end else begin
    iiCourseDir := '';
    panel.visible := true;
  end;
  Result := iiCourseDirOK;
end;

function B_LoadConfig; {---------------------------------------}
var fi: TStringList;
    i: integer;
begin
  fi := TStringList.Create;
  if FileExists (iiConfDir + SEP + 'tuxedit.cfg') then begin
    fi.LoadFromFile (iiConfDir + SEP + 'tuxedit.cfg');
    for i := 0 to fi.count-1 do begin
      if ParsStr(fi[i], 'struct') = 'common' then begin
        iiCourseDir := ParsStr (fi[i], 'coursedir');
        iiTuxBin := ParsStr (fi[i], 'tuxracer');
      end;
    end;
  end else begin
  end;
  fi.Free;
end;

function B_SaveConfig; {---------------------------------------}
var fi: TStringList;
    ss: String;
begin
  fi := TStringList.Create;
  ss := '[struct]common[coursedir]';
  if iiCourseDirOK then ss := ss + iiCourseDir;
  ss := ss + '[tuxracer]' + iiTuxBin;
  fi.Add (ss);
  fi.SaveToFile (iiConfDir + SEP + 'tuxedit.cfg');
  fi.Free;
end;

procedure B_SetMapSize; {-------------------------------}
begin
  if not iiOpen then exit;
  MMap.Height := EMap.Height * mpScale;
  MMap.Width := EMap.Width * mpScale;
  MMap2.Height := MMap.Height;
  MMap2.Width := MMap.Width;
  iiHeight := EMap.Height;
  iiWidth := EMap.Width;
end;

procedure EArr_EMap; {--------------------------------}
var p:PByteArray;
    x,y,dc:integer;
begin
  if not iiOpen then exit;
  for y:=0 to EMap.Height-1 do begin
    p:=EMap.ScanLine[y];
    for x:=0 to EMap.Width-1 do begin
      for dc:=0 to 3 do begin
        p[x*4+dc]:=EArr[y,x];
      end;
    end;
  end;
end;

procedure EMap_EArr; {--------------------------------}
var p: PByteArray;
    x, y, h, w: integer;
begin
  if not iiOpen then exit;
  for y := 0 to EMap.Height-1 do begin
    p := EMap.ScanLine[y];
    for x := 0 to EMap.Width-1 do begin
      EArr[y,x] := p[x * 4];
    end;
  end;
end;

procedure EArrRect_MMap; {---------------------------}
var x,y,dx,dy,dc,val: integer;
    p: PByteArray;
begin
  if not iiOpen then exit;
  if r.left < 0 then r.left := 0;
  if r.top < 0 then r.top := 0;
  if r.right > iiWidth-1 then r.right := iiWidth-1;
  if r.bottom > iiHeight-1 then r.bottom := iiHeight-1;

  if mpMaskMode > 0 then begin  // mask activated
    for y := r.top to r.bottom do begin
      for dy := 0 to mpScale-1 do begin
        p := MMap.Scanline[y * mpScale + dy];
        for x:= r.left to r.right do begin
          for dx :=0 to mpScale-1 do begin
            if MaskArr[y,x] > 0 then begin
              p[x * 4 * mpScale + dx * 4 + 0] := 0;
              p[x * 4 * mpScale + dx * 4 + 1] := 0;
              val := EArr [y,x] + 100;
              if val > 255 then val := 255;
              p[x * 4 * mpScale + dx * 4 + 2] := val;
            end else begin
              p[x * 4 * mpScale + dx * 4 + 0] := EArr[y,x];
              p[x * 4 * mpScale + dx * 4 + 1] := EArr[y,x];
              p[x * 4 * mpScale + dx * 4 + 2] := EArr[y,x];
            end;
          end;
        end;
      end;
    end;
  end else begin // mask not activated
    if (mpObjBright = 3)
    or (EditMode = emElev)
    or (EditMode = emCourse) then begin
      for y := r.top to r.bottom do begin
        for dy := 0 to mpScale-1 do begin
          p := MMap.Scanline[y * mpScale + dy];
          for x:= r.left to r.right do begin
            for dx :=0 to mpScale-1 do begin
              for dc:=0 to 2 do begin
                p[x * 4 * mpScale + dx * 4 + dc] := EArr[y,x];
              end;
            end;
          end;
        end;
      end;
    end else begin
      for y := r.top to r.bottom do begin
        for dy := 0 to mpScale-1 do begin
          p := MMap.Scanline[y * mpScale + dy];
          for x:= r.left to r.right do begin
            for dx :=0 to mpScale-1 do begin
              for dc:=0 to 2 do begin
                case mpObjBright of
                  1: p[x * 4 * mpScale + dx * 4 + dc] := EArr[y,x] div 4;
                  2: p[x * 4 * mpScale + dx * 4 + dc] := EArr[y,x] div 2;
                  4: p[x * 4 * mpScale + dx * 4 + dc] := EArr[y,x] div 2 + 127;
                  5: p[x * 4 * mpScale + dx * 4 + dc] := EArr[y,x] div 4 + 191;
                end;
              end;
            end;
          end;
        end;
      end;
    end;
  end;
end;

procedure EArr_MMap; {-------------------------------}
var r: TRect;
begin
  r := rect (0, 0, EMap.Width-1, EMap.Height-1);
  EArrRect_MMap (r);
end;

procedure TMap_MMap; {----------------------------------}
var r: TRect;
begin
  r:=rect(0,0,EMap.Width-1,EMap.Height-1);
  TMapRect_MMap(r);
end;

procedure TMapRect_MMap; {----------------------------------}
var x,y,dx,dy,sx,dc: integer;
    val: Byte;
    p1,p2: PByteArray;
begin
  if not iiOpen then exit;
  if r.left < 0 then r.left := 0;
  if r.top < 0 then r.top := 0;
  if r.right > EMap.Width-1 then r.right := EMap.Width-1;
  if r.bottom > EMap.Height-1 then r.bottom := EMap.Height-1;

  for y := r.top to r.bottom do begin
    p1 := TMap.Scanline[y];
    for dy:=0 to mpScale-1 do begin
      p2 := MMap.Scanline[y * mpScale + dy];
      for x := r.left to r.right do begin
        sx := x * 4 * mpScale;
        for dc := 0 to 2 do begin
          val := (EArr[y,x] * mpMix + p1[x * 4 + dc] * (10-mpMix)) div 10;
          for dx :=0 to mpScale - 1 do begin
            p2[sx + dx * 4 + dc] := val;
          end;
        end;
      end;
    end;
  end;
end;







function B_xPos; {---------------------------------------------}
begin
  Result:= iiCourseWidth / iiWidth * (iiWidth - bmX - 0.5);
end;

function B_xPixPos; {----------------------------------------}
begin
  Result:= iiWidth - (x * iiWidth / iiCourseWidth) - 0.5;
end;

function B_yPos; {---------------------------------------------}
begin
  Result:= -iiCourseLength / (iiHeight-1) * (bmY + 0.5);
end;

function B_yPixPos; {----------------------------------------}
begin
  Result:= (y * iiHeight - y) / -iiCourseLength - 0.5;
end;

function B_zPos; {---------------------------------------------}
begin
  Result := bmz / 255 * iiScale  - iiScale / 2
  - tan (iiAngle * pi / 180) * abs(-iiCourseLength / iiHeight
  * (bmy + 0.5));
end;

function B_ZPos2; {--------------------------------------}
var desc: double;
begin
  desc := tan(iiAngle * pi / 180) * abs(-iiCourseLength / iiHeight * (bmy + 0.5));
  Result := zgrey / 255 * iiScale - iiScale / 2 - desc;
end;

function B_ZGrey2; {-------------------------------------}
var desc: double;
begin
  desc := tan(iiAngle * pi / 180) * abs(-iiCourseLength / iiHeight * (bmy + 0.5));
  Result := (zpos + iiScale / 2 + desc) * 255 / iiScale;
end;




function  B_ElevFile; {----------------------------------------}
begin
  Result := iiTrackDir + SEP + 'elev.png';
end;

function  B_TerrainFile; {-------------------------------------}
begin
  Result := iiTrackDir + SEP + 'terrain.png';
end;

function  B_ObjectsFile; {-------------------------------------}
begin
  Result := iiCupDir + SEP + 'objects.tcl';
end;

function  B_OTexturesFile; {-----------------------------------}
begin
  Result := iiCupDir + SEP + 'object_textures.tcl';
end;

function  B_CourseFile; {--------------------------------------}
begin
  Result := iiTrackDir + SEP + 'course.tcl';
end;

function  B_OArrFile; {----------------------------------------}
begin
  Result := iiTrackDir + SEP + 'objects05.arr';
end;

function  B_ItemsFile; {---------------------------------------}
begin
  Result := iiTrackDir + SEP + 'items.tcl';
end;

procedure B_MakeOIdx; {------------------------------------------}
var i: integer;
    x, y: String;
    ss, struct: String;
begin
  OIdx := '';
  if OArr.Count > 0 then for i := 0 to OArr.Count-1 do begin
    ss := OArr[i];
    x := ParsStr (ss, 'x');
    y := ParsStr (ss, 'y');
    OIdx := OIdx + '[' + x + '-' + y + ']' + IntToStr(i);
  end;
end;




begin
end.
